@@ -17,9 +17,11 @@
 
 import os.path
 
+from keystoneauth1 import loading
+from keystonemiddleware import auth_token
 from oslo_config import cfg
-from oslo_config.cfg import NoSuchOptError
 from oslo_config import types
+from oslo_config.cfg import NoSuchOptError
 from oslo_log import log as logging
 from oslo_middleware import cors
 from osprofiler import opts as profiler
@@ -91,7 +93,7 @@ common_opts = [
     cfg.BoolOpt('neutron_api_insecure', default=False,
                 help="Allow to perform insecure SSL requests to neutron."),
     cfg.URIOpt('cinder_url', help='URL without the tenant segment.'),
-    cfg.StrOpt('cinder_service_type', default='volumev2',
+    cfg.StrOpt('cinder_service_type', default='volumev3',
                help='Service type to use when searching catalog.'),
     cfg.StrOpt('cinder_endpoint_type', default='publicURL',
                help='Service endpoint type to use when searching catalog.'),
@@ -175,10 +177,10 @@ common_opts = [
                help="Maximum time (in seconds) to wait for Guest Agent "
                     "'quick' requests (such as retrieving a list of "
                     "users or databases)."),
-    cfg.IntOpt('agent_call_high_timeout', default=60 * 10,
+    cfg.IntOpt('agent_call_high_timeout', default=60 * 3,
                help="Maximum time (in seconds) to wait for Guest Agent 'slow' "
                     "requests (such as restarting the database)."),
-    cfg.IntOpt('agent_replication_snapshot_timeout', default=36000,
+    cfg.IntOpt('agent_replication_snapshot_timeout', default=60 * 30,
                help='Maximum time (in seconds) to wait for taking a Guest '
                     'Agent replication snapshot.'),
     cfg.IntOpt('command_process_timeout', default=30,
@@ -186,8 +188,13 @@ common_opts = [
                     'commands to complete.'),
     # The guest_id opt definition must match the one in cmd/guest.py
     cfg.StrOpt('guest_id', default=None, help="ID of the Guest Instance."),
-    cfg.IntOpt('state_change_wait_time', default=60 * 10,
-               help='Maximum time (in seconds) to wait for a state change.'),
+    cfg.StrOpt('controller_address',
+               help='The address used to download Trove code by guest agent '
+                    'in developer mode. This address is inserted into the '
+                    'file /etc/trove/controller.conf inside the guest.'),
+    cfg.IntOpt('state_change_wait_time', default=180,
+               help='Maximum time (in seconds) to wait for database state '
+                    'change.'),
     cfg.IntOpt('state_change_poll_time', default=3,
                help='Interval between state change poll requests (seconds).'),
     cfg.IntOpt('agent_heartbeat_time', default=10,
@@ -199,6 +206,7 @@ common_opts = [
     cfg.IntOpt('num_tries', default=3,
                help='Number of times to check if a volume exists.'),
     cfg.StrOpt('volume_fstype', default='ext3',
+               choices=['ext3', 'ext4', 'xfs'],
                help='File system type used to format a volume.'),
     cfg.StrOpt('cinder_volume_type', default=None,
                help='Volume type to use when provisioning a Cinder volume.'),
@@ -241,7 +249,10 @@ common_opts = [
                'becomes required in the instance create request.'),
     cfg.StrOpt('datastore_manager', default=None,
                help='Manager class in the Guest Agent, set up by the '
-               'Taskmanager on instance provision.'),
+                    'Taskmanager on instance provision.'),
+    cfg.StrOpt('datastore_version', default=None,
+               help='The guest datastore version that is set by the '
+                    'Taskmanager during instance provision.'),
     cfg.StrOpt('block_device_mapping', default='vdb',
                help='Block device to map onto the created instance.'),
     cfg.IntOpt('server_delete_time_out', default=60,
@@ -292,9 +303,11 @@ common_opts = [
                help='The region this service is located.'),
     cfg.StrOpt('backup_runner',
                default='trove.guestagent.backup.backup_types.InnoBackupEx',
-               help='Runner to use for backups.'),
+               help='Runner to use for backups.',
+               deprecated_for_removal=True),
     cfg.DictOpt('backup_runner_options', default={},
-                help='Additional options to be passed to the backup runner.'),
+                help='Additional options to be passed to the backup runner.',
+                deprecated_for_removal=True),
     cfg.BoolOpt('verify_swift_checksum_on_restore', default=True,
                 help='Enable verification of Swift checksum before starting '
                 'restore. Makes sure the checksum of original backup matches '
@@ -303,11 +316,12 @@ common_opts = [
                 help='Require the replica volume size to be greater than '
                 'or equal to the size of the master volume '
                 'during replica creation.'),
-    cfg.StrOpt('storage_strategy', default='SwiftStorage',
+    cfg.StrOpt('storage_strategy', default='swift',
                help="Default strategy to store backups."),
     cfg.StrOpt('storage_namespace',
                default='trove.common.strategies.storage.swift',
-               help='Namespace to load the default storage strategy from.'),
+               help='Namespace to load the default storage strategy from.',
+               deprecated_for_removal=True),
     cfg.StrOpt('backup_swift_container', default='database_backups',
                help='Swift container to put backups in.'),
     cfg.BoolOpt('backup_use_gzip_compression', default=True,
@@ -334,13 +348,13 @@ common_opts = [
                default='trove.common.clients.guest_client',
                help='Client to send Guest Agent calls to.'),
     cfg.StrOpt('remote_nova_client',
-               default='trove.common.clients.nova_client',
+               default='trove.common.clients_admin.nova_client_trove_admin',
                help='Client to send Nova calls to.'),
     cfg.StrOpt('remote_neutron_client',
-               default='trove.common.clients.neutron_client',
+               default='trove.common.clients_admin.neutron_client_trove_admin',
                help='Client to send Neutron calls to.'),
     cfg.StrOpt('remote_cinder_client',
-               default='trove.common.clients.cinder_client',
+               default='trove.common.clients_admin.cinder_client_trove_admin',
                help='Client to send Cinder calls to.'),
     cfg.StrOpt('remote_swift_client',
                default='trove.common.clients.swift_client',
@@ -349,7 +363,7 @@ common_opts = [
                default='trove.common.trove_remote.trove_client',
                help='Client to send Trove calls to.'),
     cfg.StrOpt('remote_glance_client',
-               default='trove.common.clients.glance_client',
+               default='trove.common.clients_admin.glance_client_trove_admin',
                help='Client to send Glance calls to.'),
     cfg.StrOpt('exists_notification_transformer',
                help='Transformer for exists notifications.'),
@@ -372,7 +386,8 @@ common_opts = [
                          'mariadb': '7a4f82cc-10d2-4bc6-aadc-d9aacc2a3cb5'},
                 help='Unique ID to tag notification events.'),
     cfg.StrOpt('network_label_regex', default='^private$',
-               help='Regular expression to match Trove network labels.'),
+               help='Regular expression to match Trove network labels.',
+               deprecated_for_removal=True),
     cfg.StrOpt('ip_regex', default=None,
                help='List IP addresses that match this regular expression.'),
     cfg.StrOpt('black_list_regex', default=None,
@@ -427,15 +442,12 @@ common_opts = [
     cfg.IntOpt('usage_timeout', default=60 * 30,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.IntOpt('restore_usage_timeout', default=36000,
+    cfg.IntOpt('restore_usage_timeout', default=60 * 60,
                help='Maximum time (in seconds) to wait for a Guest instance '
                     'restored from a backup to become active.'),
     cfg.IntOpt('cluster_usage_timeout', default=36000,
                help='Maximum time (in seconds) to wait for a cluster to '
                     'become active.'),
-    cfg.IntOpt('timeout_wait_for_service', default=120,
-               help='Maximum time (in seconds) to wait for a service to '
-                    'become alive.'),
     cfg.StrOpt('module_aes_cbc_key', default='module_aes_cbc_key',
                help='OpenSSL aes_cbc key for module encryption.'),
     cfg.ListOpt('module_types', default=['ping', 'new_relic_license'],
@@ -464,6 +476,14 @@ common_opts = [
                help='Key (OpenSSL aes_cbc) to encrypt instance keys in DB.'),
     cfg.StrOpt('instance_rpc_encr_key',
                help='Key (OpenSSL aes_cbc) for instance RPC encryption.'),
+    cfg.StrOpt('database_service_uid', default='1001',
+               help='The UID(GID) of database service user.'),
+    cfg.ListOpt('reserved_network_cidrs', default=[],
+                help='Network CIDRs reserved for Trove guest instance '
+                     'management.'),
+    cfg.BoolOpt(
+        'online_volume_resize', default=True,
+        help='If online volume resize is supported.')
 ]
 
 
@@ -542,7 +562,7 @@ mysql_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.StrOpt('backup_strategy', default='InnoBackupEx',
+    cfg.StrOpt('backup_strategy', default='innobackupex',
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
@@ -562,28 +582,10 @@ mysql_opts = [
     cfg.IntOpt('usage_timeout', default=400,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for mysql.'),
@@ -609,6 +611,14 @@ mysql_opts = [
                help='Character length of generated passwords.',
                deprecated_name='default_password_length',
                deprecated_group='DEFAULT'),
+    cfg.StrOpt(
+        'docker_image', default='mysql',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image', default='openstacktrove/db-backup-mysql:1.0.0',
+        help='The docker image used for backup and restore.'
+    ),
 ]
 
 # Percona
@@ -651,28 +661,10 @@ percona_opts = [
     cfg.IntOpt('usage_timeout', default=450,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for percona.'),
@@ -737,22 +729,10 @@ pxc_opts = [
     cfg.IntOpt('usage_timeout', default=450,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.mysql_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.mysql_impl',
-               help='Namespace to load restore strategies from.'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'InnoBackupEx': 'InnoBackupExIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.'),
     cfg.ListOpt('ignore_users', default=['os_admin', 'root', 'clusterrepuser'],
                 help='Users to exclude when listing users.'),
     cfg.ListOpt('ignore_dbs',
@@ -816,12 +796,6 @@ redis_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default='RedisSyncReplication',
                help='Default strategy for replication.'),
     cfg.StrOpt('replication_namespace',
@@ -835,18 +809,6 @@ redis_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.StrOpt('backup_namespace',
-               default="trove.guestagent.strategies.backup.experimental."
-                       "redis_impl",
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default="trove.guestagent.strategies.restore.experimental."
-                       "redis_impl",
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('cluster_support', default=True,
                 help='Enable clusters to be created and managed.'),
     cfg.StrOpt('api_strategy',
@@ -891,12 +853,6 @@ cassandra_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental strategy based on the default backup '
-                'strategy. For strategies that do not implement incremental '
-                'backups, the runner performs full backup instead.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('backup_strategy', default="NodetoolSnapshot",
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
@@ -910,18 +866,6 @@ cassandra_opts = [
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.StrOpt('backup_namespace',
-               default="trove.guestagent.strategies.backup.experimental."
-               "cassandra_impl",
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default="trove.guestagent.strategies.restore.experimental."
-               "cassandra_impl",
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for Cassandra.'),
@@ -1000,12 +944,6 @@ couchbase_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/couchbase',
@@ -1016,18 +954,6 @@ couchbase_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-               'couchbase_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-               'couchbase_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
@@ -1064,12 +990,6 @@ mongodb_opts = [
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/mongodb',
@@ -1107,18 +1027,6 @@ mongodb_opts = [
                'mongodb.guestagent.MongoDbGuestAgentStrategy',
                help='Class that implements datastore-specific Guest Agent API '
                     'logic.'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-                       'mongo_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-                       'mongo_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.PortOpt('mongodb_port', default=27017,
                 help='Port for mongod and mongos instances.'),
     cfg.PortOpt('configsvr_port', default=27019,
@@ -1147,6 +1055,15 @@ postgresql_group = cfg.OptGroup(
     'postgresql', title='PostgreSQL options',
     help="Oslo option group for the PostgreSQL datastore.")
 postgresql_opts = [
+    cfg.StrOpt(
+        'docker_image', default='postgres',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image',
+        default='openstacktrove/db-backup-postgresql:1.0.0',
+        help='The docker image used for backup and restore.'
+    ),
     cfg.BoolOpt('icmp', default=False,
                 help='Whether to permit ICMP.',
                 deprecated_for_removal=True),
@@ -1160,20 +1077,18 @@ postgresql_opts = [
                      'if trove_security_groups_support is True).'),
     cfg.PortOpt('postgresql_port', default=5432,
                 help='The TCP port the server listens on.'),
-    cfg.StrOpt('backup_strategy', default='PgBaseBackup',
+    cfg.StrOpt('backup_strategy', default='pg_basebackup',
                help='Default strategy to perform backups.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'PgBaseBackup': 'PgBaseBackupIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
-    cfg.StrOpt('replication_strategy',
-               default='PostgresqlReplicationStreaming',
-               help='Default strategy for replication.'),
-    cfg.StrOpt('replication_namespace',
-               default='trove.guestagent.strategies.replication.experimental.'
-                       'postgresql_impl',
-               help='Namespace to load replication strategies from.'),
+    cfg.StrOpt(
+        'replication_strategy',
+        default='PostgresqlReplicationStreaming',
+        help='Default strategy for replication.'
+    ),
+    cfg.StrOpt(
+        'replication_namespace',
+        default='trove.guestagent.strategies.replication.postgresql',
+        help='Namespace to load replication strategies from.'
+    ),
     cfg.StrOpt('mount_point', default='/var/lib/postgresql',
                help="Filesystem path for mounting "
                "volumes if volume support is enabled."),
@@ -1186,18 +1101,10 @@ postgresql_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-               'postgresql_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-               'postgresql_impl',
-               help='Namespace to load restore strategies from.'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb'),
-    cfg.ListOpt('ignore_users', default=['os_admin', 'postgres', 'root']),
+    cfg.ListOpt('ignore_users', default=['os_admin', 'postgres']),
     cfg.ListOpt('ignore_dbs', default=['os_admin', 'postgres']),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
@@ -1246,16 +1153,6 @@ couchdb_opts = [
                help='Default strategy to perform backups.'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
-    cfg.StrOpt('backup_namespace', default='trove.guestagent.strategies'
-               '.backup.experimental.couchdb_impl',
-               help='Namespace to load backup strategies from.'),
-    cfg.StrOpt('restore_namespace', default='trove.guestagent.strategies'
-               '.restore.experimental.couchdb_impl',
-               help='Namespace to load restore strategies from.'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.BoolOpt('root_on_create', default=False,
                 help='Enable the automatic creation of the root user for the '
                 'service during instance-create. The generated password for '
@@ -1301,10 +1198,6 @@ vertica_opts = [
                      'if trove_security_groups_support is True).'),
     cfg.StrOpt('backup_strategy', default=None,
                help='Default strategy to perform backups.'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.StrOpt('replication_strategy', default=None,
                help='Default strategy for replication.'),
     cfg.StrOpt('mount_point', default='/var/lib/vertica',
@@ -1315,9 +1208,11 @@ vertica_opts = [
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
     cfg.StrOpt('backup_namespace', default=None,
-               help='Namespace to load backup strategies from.'),
+               help='Namespace to load backup strategies from.',
+               deprecated_for_removal=True),
     cfg.StrOpt('restore_namespace', default=None,
-               help='Namespace to load restore strategies from.'),
+               help='Namespace to load restore strategies from.',
+               deprecated_for_removal=True),
     cfg.IntOpt('readahead_size', default=2048,
                help='Size(MB) to be set as readahead_size for data volume'),
     cfg.BoolOpt('cluster_support', default=True,
@@ -1385,22 +1280,6 @@ db2_opts = [
                 'service during instance-create. The generated password for '
                 'the root user is immediately returned in the response of '
                 "instance-create as the 'password' field."),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental.'
-                       'db2_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental.'
-                       'db2_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.DictOpt('backup_incremental_strategy', default={},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental, the runner will use the default full backup.'),
     cfg.ListOpt('ignore_users', default=['PUBLIC', 'DB2INST1']),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
@@ -1430,21 +1309,14 @@ mariadb_opts = [
                 help='List of UDP ports and/or port ranges to open '
                      'in the security group (only applicable '
                      'if trove_security_groups_support is True).'),
-    cfg.StrOpt('backup_namespace',
-               default='trove.guestagent.strategies.backup.experimental'
-                       '.mariadb_impl',
-               help='Namespace to load backup strategies from.',
-               deprecated_name='backup_namespace',
-               deprecated_group='DEFAULT'),
-    cfg.StrOpt('backup_strategy', default='MariaBackup',
+    cfg.StrOpt('backup_strategy', default='mariabackup',
                help='Default strategy to perform backups.',
                deprecated_name='backup_strategy',
                deprecated_group='DEFAULT'),
     cfg.StrOpt('replication_strategy', default='MariaDBGTIDReplication',
                help='Default strategy for replication.'),
     cfg.StrOpt('replication_namespace',
-               default='trove.guestagent.strategies.replication.experimental'
-               '.mariadb_gtid',
+               default='trove.guestagent.strategies.replication.mariadb_gtid',
                help='Namespace to load replication strategies from.'),
     cfg.StrOpt('mount_point', default='/var/lib/mysql',
                help="Filesystem path for mounting "
@@ -1457,25 +1329,10 @@ mariadb_opts = [
     cfg.IntOpt('usage_timeout', default=400,
                help='Maximum time (in seconds) to wait for a Guest to become '
                     'active.'),
-    cfg.StrOpt('restore_namespace',
-               default='trove.guestagent.strategies.restore.experimental'
-                       '.mariadb_impl',
-               help='Namespace to load restore strategies from.',
-               deprecated_name='restore_namespace',
-               deprecated_group='DEFAULT'),
     cfg.BoolOpt('volume_support', default=True,
                 help='Whether to provision a Cinder volume for datadir.'),
     cfg.StrOpt('device_path', default='/dev/vdb',
                help='Device path for volume if volume support is enabled.'),
-    cfg.DictOpt('backup_incremental_strategy',
-                default={'MariaBackup':
-                         'MariaBackupIncremental'},
-                help='Incremental Backup Runner based on the default '
-                'strategy. For strategies that do not implement an '
-                'incremental backup, the runner will use the default full '
-                'backup.',
-                deprecated_name='backup_incremental_strategy',
-                deprecated_group='DEFAULT'),
     cfg.StrOpt('root_controller',
                default='trove.extensions.common.service.DefaultRootController',
                help='Root controller implementation for mysql.'),
@@ -1519,6 +1376,15 @@ mariadb_opts = [
                help='Character length of generated passwords.',
                deprecated_name='default_password_length',
                deprecated_group='DEFAULT'),
+    cfg.StrOpt(
+        'docker_image', default='mariadb',
+        help='Database docker image.'
+    ),
+    cfg.StrOpt(
+        'backup_docker_image',
+        default='openstacktrove/db-backup-mariadb:1.0.0',
+        help='The docker image used for backup and restore.'
+    ),
 ]
 
 # RPC version groups
@@ -1552,7 +1418,14 @@ network_opts = [
         help='ID of the Neutron public network to create floating IP for the '
              'public trove instance. If not given, Trove will try to query '
              'all the public networks and use the first one in the list.'
-    )
+    ),
+    cfg.BoolOpt(
+        'enable_access_check', default=True,
+        help='Check if the user provided network is associated with router. '
+             'This is needed for the instance initialization. The check is '
+             'also necessary when creating public facing instance. A scenario '
+             'to set this option False is when using Neutron provider '
+             'network.')
 ]
 
 service_credentials_group = cfg.OptGroup(
@@ -1638,6 +1511,24 @@ profiler.set_defaults(CONF)
 logging.register_options(CONF)
 
 
+def list_opts():
+    keystone_middleware_opts = auth_token.list_opts()
+    keystone_loading_opts = [(
+        'keystone_authtoken', loading.get_auth_plugin_conf_options('password')
+    )]
+
+    trove_opts = [
+        (None, path_opts + versions_opts + common_opts),
+        ('database', database_opts),
+        (mysql_group, mysql_opts),
+        (mariadb_group, mariadb_opts),
+        (network_group, network_opts),
+        (service_credentials_group, service_credentials_opts),
+    ]
+
+    return keystone_middleware_opts + keystone_loading_opts + trove_opts
+
+
 def custom_parser(parsername, parser):
     CONF.register_cli_opt(cfg.SubCommandOpt(parsername, handler=parser))
 
